/*
 *				**** Get Info ****
 *
 * This is a sample program to help students learn how to use
 * various packages that are available on Bill Pringle's website
 */

/* standard includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* include file for Bill Pringle's packages */
#include "prodlist.h"
#include "read_line.h"
#include "linklist.h"

/* include file for this program */
#include "getinfo.h"

#include "stuff.h"

/* prototypes for local routines */
void enter_stuff(ProdPtrList *ent);	/* enter entries interactively */
void display_entries(ProdPtrList *ent); /* display all entries */
int get_option(char *buf);		/* promput user for option */
void index_name(ProdPtrList *list, LinkList *ndx);	/* build name index */
ProdPtrList *find_name(ProdPtrList *list, LinkList *ndx);	/* find using name index */
void save_entries(ProdPtrList *list);
void load_entries(ProdPtrList *list);
void append_entries(ProdPtrList *list);

int main(int argc, char *argv[])
{
	char buf[BUFSIZ+1];
   ProdPtrList *stuff;
   LinkList *name_ndx=NULL;	/* index for names */
   struct stuff *ent;
   int i, opt, running=1;

   buf[BUFSIZ] = '\0';	/* prevent buffer overflow */

   printf("GetInfo Program, version %s\n", VERSION);

   /* create list for the collection */
   stuff = create_ptr_prodlist(100);	/* starting size */
   name_ndx = create_link_list();	/* used to index names */

   while (running)
   {
   	opt = get_option(buf);
      if (*buf == 'q') opt = OPT_DONE;	/* special shortcut */
      switch(opt)
      {
      case OPT_ENTER:
      	enter_stuff(stuff);
      	break;
      case OPT_DISPLAY:
      	display_entries(stuff);
         break;
      case OPT_NDX_NAME:
			index_name(stuff, name_ndx);	/* build name index */
      	break;
      case OPT_SRCH_NAME:
      	break;
      case OPT_SAVE:
      	save_entries(stuff);
      	break;
      case OPT_LOAD:
      	load_entries(stuff);
         break;
      case OPT_APPEND:
      	append_entries(stuff);
      	break;
      case OPT_DONE:
      	printf("\nAre you sure: (y or n): ");
         gets(buf);
			if (*buf == 'y') running=0;
         break;
      default:
      	printf("Invalid option (%d)\n", opt);
      }
   }

   return 0;
}

void display_entries(ProdPtrList *stuff)
{
   struct stuff *ent;
	int i, j, n1, n2;
   char buf[BUFSIZ+1];

   buf[BUFSIZ] = '\0'; /* prevent buffer overflow */

   /* get range to display */
   printf("Enter first entry to display (0-%d): ", stuff->nitems-1);
   gets(buf);
   n1 = atoi(buf);
   if (n1 < 0 || *buf == '\0') return;	/* bad value - return */
   printf("Enter last entry to display (%d-%d): ", n1, stuff->nitems-1);
   gets(buf);
   n2 = atoi(buf);

   /* display requested entries */
   for (i=n1,j=0; i < n2; i++)
   {
   	ent = (struct stuff *)stuff->item[i];	/* get item */
      display_stuff(ent);
      if ((++j % 20) == 0)
      {
      	printf("Press ENTER to continue: ");
         gets(buf);
         if (*buf == 'q') break;
      }
	}
}

void enter_stuff(ProdPtrList *stuff)
{
	char buf[BUFSIZ+1];
   struct stuff *ent;

   buf[BUFSIZ] = '\0';	/* prevent buffer overflow */

   /* get list of entries from user */
   while (1)
   {
    	printf("Enter ID, or ENTER to quit: ");
      read_line(buf, stdin);
      if (*buf == '\0') break;	/* done entering info */
      /* create entry and fill in fields */
      ent = create_stuff();
      ent->id = atoi(buf);
      printf("Enter last name: ");
      read_line(buf, stdin);
      ent->lname = strdup(buf);	/* last name */
      printf("Enter first name: ");
      read_line(buf, stdin);
      ent->fname = strdup(buf);
      printf("Enter phone number: ");
      read_line(buf, stdin);
      ent->phone = strdup(buf);

      /* now add entry to list */
      add_ptr_prodlist(stuff, ent);
   }
}

int get_option(char *buf)
{
	int opt;

   printf("\nOptions are:\n");
   printf("%2d. Enter entries\n", OPT_ENTER);
   printf("%2d. Display entries\n", OPT_DISPLAY);
   printf("%2d. Build Name index\n", OPT_NDX_NAME);
   printf("%2d. Search by Name\n", OPT_SRCH_NAME);
   printf("%2d. Save entries\n", OPT_SAVE);
   printf("%2d. Load from file\n", OPT_LOAD);
   printf("%2d. Append from file\n", OPT_APPEND);
	printf("%2d. Quit\n", OPT_DONE);
   printf("\nEnter option: ");
   gets(buf);
   opt = atoi(buf);

   return opt;
}

void index_name(ProdPtrList *list, LinkList *ndx)
{	/* build name index */
	printf("Not implemented yet.\n");
}

ProdPtrList *find_name(ProdPtrList *list, LinkList *ndx)
{	/* find using name index */
	printf("Not implemented yet.\n");
	return NULL;
}

void save_entries(ProdPtrList *list)
{
	char buf[BUFSIZ];
   FILE *fp;
   int i;
   struct stuff *ent;

   printf("\nEnter name of output file: ");
   gets(buf);
   if ((fp = fopen(buf, "w")) == NULL)
   {
   	printf("Can't open output file (%s)\n", buf);
      return;
   }

   save_stuff(fp, list);
   fclose(fp);
}

void load_entries(ProdPtrList *list)
{
	char buf[BUFSIZ];
   FILE *fp;

   printf("Enter name of input file: ");
   gets(buf);
   if (*buf == '\0') return; /* no file - return quietly */
   if ((fp = fopen(buf, "r")) == NULL)
   {
   	printf("Can't open input file (%s)\n", buf);
      return;
   }

   load_stuff(fp, list);
}

void append_entries(ProdPtrList *list)
{
	char buf[BUFSIZ];
   FILE *fp;

   printf("Enter name of input file: ");
   gets(buf);
   if (*buf == '\0') return;	/* no file - return quietly */
   if ((fp = fopen(buf, "r")) == NULL)
   {
   	printf("Can't open input file (%s)\n", buf);
      return;
   }

   append_stuff(fp, list);
}


