/*
 *				**** STUFF ****
 *
 * This file contains routines that deal with stuff
 */

#include <stdio.h>
#include <stdlib.h>

#include "read_line.h"
#include "split_flds.h"

#include "stuff.h"

struct stuff *create_stuff(void)
{
	struct stuff *ret;

   ret = (struct stuff *)malloc(sizeof(*ret));
   if (ret != NULL) memset(ret, '\0', sizeof(*ret));

   return ret;
}

void free_stuff(struct stuff *ent)
{
	if (ent == NULL) return; /* bad argument */

   /* free up any memory associated with this entry */
   if (ent->lname != NULL) free(ent->lname);
   if (ent->fname != NULL) free(ent->fname);
   if (ent->phone != NULL) free(ent->phone);

   /* now free up the actual entry */
   free(ent);
}

void display_stuff(struct stuff *ent)
{
      printf("ID=%d, Name=%s, %s, Phone=%s\n",
      	ent->id, ent->lname, ent->fname, ent->phone);
}

void save_stuff(FILE *fp, ProdPtrList *list)
{
	struct stuff *ent;
   int i;
   
   for (i=0; i < list->nitems; i++)
   { /* loop through list */
   	ent = (struct stuff *)list->item[i];	/* get entry */
      write_stuff(fp, ent);
   }
}

void write_stuff(FILE *fp, struct stuff *ent)
{
      fprintf(fp, "%d|%s|%s|%s\n", ent->id, ent->lname, ent->fname, ent->phone);
}

void load_stuff(FILE *fp, ProdPtrList *list)
{
	int i;
   struct stuff *ent;

   if (fp == NULL || fp == NULL) return;	/* bad args */

   for (i=0; i < list->nitems; i++)
   { /* free up any existing entries */
   	ent = (struct stuff *)list->item[i];
      if (ent != NULL)
      {
      	free_stuff(ent);
         list->item[i] = NULL;
      }
   }
   list->nitems = 0;	/* list is now empty */

   append_stuff(fp, list);	/* now load it */
}

void append_stuff(FILE *fp, ProdPtrList *list)
{
   struct stuff *ent;
	char buf[BUFSIZ+1];
   char *flds[STUFF_MAXFLDS];
   int nflds;

   buf[BUFSIZ] = '\0';

	while (read_line(buf, fp) == 0)
   {
   	nflds = split_flds(buf, flds, '|', 0, STUFF_MAXFLDS);
      if (nflds != STUFF_MAXFLDS-1)
      {
      	fprintf(stderr, "Invalid record - %d fields instead of %d\n", nflds, STUFF_MAXFLDS-1);
         return;
      }
      if ((ent = create_stuff()) == NULL)
      {
      	fprintf(stderr, "Error allocating memory for structure!\n");
         return;
      }
      /* now fill in structure */
      ent->id = atoi(flds[0]);
      ent->lname = strdup(flds[1]);
      ent->fname = strdup(flds[2]);
      ent->phone = strdup(flds[3]);
		add_ptr_prodlist(list, ent);	/* append to collection */
   }
   fclose(fp);
}

